"""
Simple example showing how to use the Enhanced Hybrid Retriever
"""

import sys
import os
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

from bm25_chroma.hybrid_retriever import HybridRetriever



def main():
    # Initialize retriever
    retriever = HybridRetriever(
        chroma_path="./test_db",
        collection_name="simple_docs"
    )
    
    # Add some documents
    documents = [
        "Machine learning is a powerful tool for data analysis and prediction.",
        "Natural language processing helps computers understand human language.",
        "Deep learning neural networks can learn complex patterns from data.",
        "Computer vision enables machines to interpret and analyze visual information.",
        "Reinforcement learning trains agents through trial and error."
    ]
    
    # Add documents in unified mode
    stats = retriever.add_documents_batch(
        documents,
        mode="unified",
        chroma_batch_size=8,
        show_progress=True
    )
    
    print(f"\nAdded {stats['total_documents']} documents in {stats['total_time_seconds']:.2f}s")
    
    # Test search
    query = "machine learning data analysis"
    print(f"\nSearching for: '{query}'")
    
    # Hybrid search
    results = retriever.hybrid_search(query, top_k=3)
    
    print("\nTop results:")
    for i, (doc_id, score, metadata) in enumerate(results, 1):
        text = metadata['text']
        print(f"{i}. {doc_id}: {score:.3f}")
        print(f"   {text}")
    
    # Show system stats
    print(f"\nSystem stats:")
    stats = retriever.get_system_stats()
    for key, value in stats.items():
        print(f"  {key}: {value}")


if __name__ == "__main__":
    main()


"""
Complete Brown Corpus demo with mode comparison
"""

import nltk
from nltk.corpus import brown
from nltk.tokenize.treebank import TreebankWordDetokenizer
import re
import sys
import os
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
from bm25_chroma.hybrid_retriever import HybridRetriever

# Download Brown corpus if needed
nltk.download('brown', quiet=True)

def get_brown_docs(num_docs=50):
    """Extract clean document texts from Brown corpus"""
    fileids = brown.fileids()[:num_docs]
    docs = []
    detokenizer = TreebankWordDetokenizer()
    
    for fileid in fileids:
        paragraphs = brown.paras(fileid)
        doc_paragraphs = []
        
        for paragraph in paragraphs:
            para_lines = []
            
            for sentence in paragraph:
                # Convert quotes
                processed_sent = []
                for token in sentence:
                    if token == '``':
                        processed_sent.append('"')
                    elif token == "''":
                        processed_sent.append('"')
                    else:
                        processed_sent.append(token)
                
                # Detokenize and clean
                detokenized_line = detokenizer.detokenize(processed_sent)
                
                # Fix spacing issues
                detokenized_line = re.sub(r'\s+([.!?])\s+', r'\1 ', detokenized_line)
                detokenized_line = re.sub(r'"\s+([.!?])', r'"\1', detokenized_line)
                detokenized_line = re.sub(r'"\s+([^"]+?)\s+"', r'"\1"', detokenized_line)
                detokenized_line = re.sub(r'^\s*"\s+', r'"', detokenized_line)
                detokenized_line = re.sub(r'\s+"\s*$', r'"', detokenized_line)
                detokenized_line = re.sub(r'([.!?])([A-Z])', r'\1 \2', detokenized_line)
                detokenized_line = re.sub(r'([,:;])([A-Za-z])', r'\1 \2', detokenized_line)
                detokenized_line = re.sub(r'\s+', ' ', detokenized_line).strip()
                
                para_lines.append(detokenized_line)
            
            paragraph_text = ' '.join(para_lines)
            doc_paragraphs.append(paragraph_text)
        
        doc_text = '\n\n'.join(doc_paragraphs)
        if len(doc_text.strip()) >= 100:  # Filter short docs
            docs.append(doc_text)
    
    return docs[:num_docs]


def main():
    print("Brown Corpus Hybrid Retriever Demo")
    
    # Get Brown corpus documents
    print("Loading Brown corpus...")
    brown_docs = get_brown_docs(50)
    print(f"Loaded {len(brown_docs)} documents")
    
    # Initialize retriever
    retriever = HybridRetriever(
        chroma_path="./brown_db",
        collection_name="brown_corpus"
    )
    
    # Test unified mode
    print("\n" + "="*50)
    print("TESTING UNIFIED MODE")
    print("="*50)
    
    unified_stats = retriever.add_documents_batch(
        brown_docs,
        doc_ids=[f"brown_{i:04d}" for i in range(len(brown_docs))],
        mode="unified",
        chroma_batch_size=8,
        show_progress=True
    )
    
    print(f"\nUnified Results:")
    print(f"  Time: {unified_stats['total_time_seconds']:.2f}s")
    print(f"  Speed: {unified_stats['docs_per_second']:.1f} docs/sec")
    
    # Test search functionality
    print("\n" + "="*50)
    print("TESTING SEARCH")
    print("="*50)
    
    test_queries = [
        "government politics",
        "economic development", 
        "social relationships"
    ]
    
    for query in test_queries:
        print(f"\nQuery: '{query}'")
        
        # BM25 search
        bm25_results = retriever.search_bm25(query, top_k=3)
        print(f"  BM25 ({len(bm25_results)} results):")
        for i, (doc_id, score) in enumerate(bm25_results[:3], 1):
            text = retriever.chunk_cache.get(doc_id, "")[:80]
            print(f"    {i}. {doc_id}: {score:.3f} - {text}...")
        
        # Vector search
        vector_results = retriever.search_vector(query, top_k=3)
        print(f"  Vector ({len(vector_results)} results):")
        for i, (doc_id, score) in enumerate(vector_results[:3], 1):
            text = retriever.chunk_cache.get(doc_id, "")[:80]
            print(f"    {i}. {doc_id}: {score:.3f} - {text}...")
        
        # Hybrid search
        hybrid_results = retriever.hybrid_search(query, top_k=3, use_rrf=True)
        print(f"  Hybrid ({len(hybrid_results)} results):")
        for i, (doc_id, score, metadata) in enumerate(hybrid_results[:3], 1):
            text = metadata.get('text', '')[:80]
            print(f"    {i}. {doc_id}: {score:.3f} - {text}...")
    
    # System statistics
    print("\n" + "="*50)
    print("FINAL STATS")
    print("="*50)
    
    stats = retriever.get_system_stats()
    for key, value in stats.items():
        print(f"  {key}: {value}")
    
    print(f"\nDemo complete! Indexed {stats['chunks']} Brown corpus documents.")


if __name__ == "__main__":
    main()


"""
Test that examples run without errors
"""
import subprocess
import sys
import os
from pathlib import Path
import subprocess
import sys
import os
from pathlib import Path

def test_basic_usage():
    """Test that basic_usage.py runs successfully"""
    # Go up one level to root, then into examples
    root_dir = Path(__file__).parent.parent
    result = subprocess.run([
        sys.executable, "examples/basic_usage.py"
    ], capture_output=True, text=True, cwd=root_dir)
    
    assert result.returncode == 0, f"basic_usage.py failed: {result.stderr}"
    assert "System stats:" in result.stdout
    assert "Top results:" in result.stdout

def test_brown_corpus_demo():
    """Test that brown_corpus_demo.py runs successfully"""
    root_dir = Path(__file__).parent.parent
    result = subprocess.run([
        sys.executable, "examples/brown_corpus_demo.py"
    ], capture_output=True, text=True, cwd=root_dir)
    
    assert result.returncode == 0, f"brown_corpus_demo.py failed: {result.stderr}"
    assert "Demo complete!" in result.stdout
    assert "TESTING SEARCH" in result.stdout

# Keep the existing test functions but update the internal imports
def test_bm25_basic():
    """Test BM25 basic functionality"""
    # Add parent directory to path for direct imports
    sys.path.insert(0, str(Path(__file__).parent.parent))
    try:
        # Changed from: from bm25_chroma.bm25 import BM25
        from bm25_chroma.bm25 import BM25
        bm25 = BM25()
        bm25.add_chunk("doc1", "machine learning algorithms")
        bm25.add_chunk("doc2", "natural language processing")
        results = bm25.search("machine learning")
        assert len(results) > 0
        assert results[0][0] == "doc1"
    finally:
        # Clean up sys.path
        sys.path.remove(str(Path(__file__).parent.parent))

def test_hybrid_retriever_basic():
    """Test HybridRetriever basic functionality"""
    sys.path.insert(0, str(Path(__file__).parent.parent))
    try:
        # Changed from: from hybrid_retriever import HybridRetriever
        from bm25_chroma.hybrid_retriever import HybridRetriever
        import tempfile
        import shutil
        import time
        
        # Use temp directory
        temp_dir = tempfile.mkdtemp()
        try:
            retriever = HybridRetriever(
                chroma_path=temp_dir,
                collection_name="test_collection"
            )
            docs = ["machine learning", "deep learning"]
            stats = retriever.add_documents_batch(docs, show_progress=False)
            assert stats['total_documents'] == 2
            assert stats['docs_per_second'] > 0
            results = retriever.hybrid_search("learning", top_k=2)
            assert len(results) == 2
        finally:
            # Close any open connections and wait a bit for file handles to release
            try:
                if 'retriever' in locals():
                    # Try to close chroma client if it exists
                    if hasattr(retriever, 'chroma_client'):
                        # Force garbage collection on the client
                        delattr(retriever, 'chroma_client')
                time.sleep(0.1)  # Small delay to allow file handles to close
            except:
                pass
            # Try multiple times to remove the directory
            for i in range(3):
                try:
                    shutil.rmtree(temp_dir)
                    break
                except PermissionError:
                    if i < 2:  # Don't sleep on the last attempt
                        time.sleep(0.1 * (i + 1))  # Increasing delay
                    else:
                        # If all attempts fail, don't let this stop the test
                        pass
    finally:
        sys.path.remove(str(Path(__file__).parent.parent))

if __name__ == "__main__":
    test_basic_usage()
    test_brown_corpus_demo() 
    test_bm25_basic()
    test_hybrid_retriever_basic()
    print("All tests passed!")